package at.ac.uibk.gitsearch.repository.search;

import static org.elasticsearch.index.query.QueryBuilders.queryStringQuery;

import at.ac.uibk.gitsearch.domain.SavedSearches;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.elasticsearch.core.ElasticsearchRestTemplate;
import org.springframework.data.elasticsearch.core.SearchHit;
import org.springframework.data.elasticsearch.core.query.NativeSearchQuery;
import org.springframework.data.elasticsearch.repository.ElasticsearchRepository;

/**
 * Spring Data Elasticsearch repository for the {@link SavedSearches} entity.
 */
public interface SavedSearchesSearchRepository
    extends ElasticsearchRepository<SavedSearches, Long>, SavedSearchesSearchRepositoryInternal {}

interface SavedSearchesSearchRepositoryInternal {
    Page<SavedSearches> search(String query, Pageable pageable);
}

class SavedSearchesSearchRepositoryInternalImpl implements SavedSearchesSearchRepositoryInternal {

    private final ElasticsearchRestTemplate elasticsearchTemplate;

    SavedSearchesSearchRepositoryInternalImpl(ElasticsearchRestTemplate elasticsearchTemplate) {
        this.elasticsearchTemplate = elasticsearchTemplate;
    }

    @Override
    public Page<SavedSearches> search(String query, Pageable pageable) {
        NativeSearchQuery nativeSearchQuery = new NativeSearchQuery(queryStringQuery(query));
        nativeSearchQuery.setPageable(pageable);
        List<SavedSearches> hits = elasticsearchTemplate
            .search(nativeSearchQuery, SavedSearches.class)
            .map(SearchHit::getContent)
            .stream()
            .collect(Collectors.toList());

        return new PageImpl<>(hits, pageable, hits.size());
    }
}
